@@ -94,6 +94,7 @@ typedef struct rd_kafka_lwtopic_s rd_kafka_lwtopic_t;
 #include "rdkafka_metadata.h"
 #include "rdkafka_mock.h"
 #include "rdkafka_partition.h"
+#include "rdkafka_assignment.h"
 #include "rdkafka_coord.h"
 #include "rdkafka_mock.h"
 
@@ -174,8 +175,14 @@ typedef enum {
         /**< commit_transaction() has been called and all outstanding
          *   messages, partitions, and offsets have been sent. */
         RD_KAFKA_TXN_STATE_COMMITTING_TRANSACTION,
+        /**< Transaction successfully committed but application has not made
+         *   a successful commit_transaction() call yet. */
+        RD_KAFKA_TXN_STATE_COMMIT_NOT_ACKED,
         /**< abort_transaction() has been called. */
         RD_KAFKA_TXN_STATE_ABORTING_TRANSACTION,
+        /**< Transaction successfully aborted but application has not made
+         *   a successful abort_transaction() call yet. */
+        RD_KAFKA_TXN_STATE_ABORT_NOT_ACKED,
         /**< An abortable error has occurred. */
         RD_KAFKA_TXN_STATE_ABORTABLE_ERROR,
         /* A fatal error has occured. */
@@ -196,7 +203,9 @@ rd_kafka_txn_state2str (rd_kafka_txn_state_t state) {
                 "InTransaction",
                 "BeginCommit",
                 "CommittingTransaction",
+                "CommitNotAcked",
                 "AbortingTransaction",
+                "AbortedNotAcked",
                 "AbortableError",
                 "FatalError"
         };
@@ -388,6 +397,10 @@ struct rd_kafka_s {
                                                           *   take action when
                                                           *   the broker state
                                                           *   changes. */
+                rd_bool_t txn_requires_epoch_bump; /**< Coordinator epoch bump
+                                                    *   required to recover from
+                                                    *   idempotent producer
+                                                    *   fatal error. */
 
                 /**< Blocking transactional API application call
                  *   currently being handled, its state, reply queue and how
@@ -470,6 +483,12 @@ struct rd_kafka_s {
                 /**< Partitions added and registered to transaction. */
                 rd_kafka_toppar_tqhead_t txn_rktps;
 
+                /**< Number of messages that failed delivery.
+                 *   If this number is >0 on transaction_commit then an
+                 *   abortable transaction error will be raised.
+                 *   Is reset to zero on each begin_transaction(). */
+                rd_atomic64_t txn_dr_fails;
+
                 /**< Current transaction error. */
                 rd_kafka_resp_err_t txn_err;
 
@@ -486,6 +505,23 @@ struct rd_kafka_s {
                 rd_kafka_timer_t    txn_coord_tmr;
         } rk_eos;
 
+
+        /**
+         * Consumer state
+         *
+         * @locality rdkafka main thread
+         * @locks_required none
+         */
+        struct {
+                /** Application consumer queue for messages, events and errors.
+                 *  (typically points to rkcg_q) */
+                rd_kafka_q_t *q;
+                /** Current assigned partitions through assign() et.al. */
+                rd_kafka_assignment_t assignment;
+                /** Waiting for this number of commits to finish. */
+                int wait_commit_cnt;
+        } rk_consumer;
+
         /**<
          * Coordinator cache.
          *
@@ -497,7 +533,6 @@ struct rd_kafka_s {
         TAILQ_HEAD(, rd_kafka_coord_req_s) rk_coord_reqs; /**< Coordinator
                                                            *   requests */
 
-	const rd_kafkap_bytes_t *rk_null_bytes;
 
 	struct {
 		mtx_t lock;       /* Protects acces to this struct */
@@ -521,6 +556,9 @@ struct rd_kafka_s {
                                    *   before rd_kafka_new() returns. */
         mtx_t rk_init_lock;       /**< Lock for rk_init_wait and _cmd */
 
+        rd_ts_t rk_ts_created;    /**< Timestamp (monotonic clock) of
+                                   *   rd_kafka_t creation. */
+
         /**
          * Background thread and queue,
          * enabled by setting `background_event_cb()`.
@@ -707,21 +745,28 @@ rd_kafka_curr_msgs_cnt (rd_kafka_t *rk) {
 /**
  * @brief Wait until \p tspec for curr_msgs to reach 0.
  *
- * @returns remaining curr_msgs
+ * @returns rd_true if zero is reached, or rd_false on timeout.
+ *          The remaining messages are returned in \p *curr_msgsp
  */
-static RD_INLINE RD_UNUSED int
-rd_kafka_curr_msgs_wait_zero (rd_kafka_t *rk, const struct timespec *tspec) {
-        int cnt;
+static RD_INLINE RD_UNUSED rd_bool_t
+rd_kafka_curr_msgs_wait_zero (rd_kafka_t *rk, int timeout_ms,
+                              unsigned int *curr_msgsp) {
+        unsigned int cnt;
+        struct timespec tspec;
+
+        rd_timeout_init_timespec(&tspec, timeout_ms);
 
         mtx_lock(&rk->rk_curr_msgs.lock);
         while ((cnt = rk->rk_curr_msgs.cnt) > 0) {
-                cnd_timedwait_abs(&rk->rk_curr_msgs.cnd,
-                                  &rk->rk_curr_msgs.lock,
-                                  tspec);
+                if (cnd_timedwait_abs(&rk->rk_curr_msgs.cnd,
+                                      &rk->rk_curr_msgs.lock,
+                                      &tspec) == thrd_timedout)
+                        break;
         }
         mtx_unlock(&rk->rk_curr_msgs.lock);
 
-        return cnt;
+        *curr_msgsp = cnt;
+        return cnt == 0;
 }
 
 void rd_kafka_destroy_final (rd_kafka_t *rk);
@@ -812,6 +857,8 @@ const char *rd_kafka_purge_flags2str (int flags);
 #define RD_KAFKA_DBG_ADMIN          0x4000
 #define RD_KAFKA_DBG_EOS            0x8000
 #define RD_KAFKA_DBG_MOCK           0x10000
+#define RD_KAFKA_DBG_ASSIGNOR       0x20000
+#define RD_KAFKA_DBG_CONF           0x40000
 #define RD_KAFKA_DBG_ALL            0xfffff
 #define RD_KAFKA_DBG_NONE           0x0
 
@@ -909,8 +956,8 @@ rd_kafka_fatal_error_code (rd_kafka_t *rk) {
 extern rd_atomic32_t rd_kafka_thread_cnt_curr;
 extern char RD_TLS rd_kafka_thread_name[64];
 
-void rd_kafka_set_thread_name (const char *fmt, ...);
-void rd_kafka_set_thread_sysname (const char *fmt, ...);
+void rd_kafka_set_thread_name (const char *fmt, ...) RD_FORMAT(printf, 1, 2);
+void rd_kafka_set_thread_sysname (const char *fmt, ...) RD_FORMAT(printf, 1, 2);
 
 int rd_kafka_path_is_dir (const char *path);
 rd_bool_t rd_kafka_dir_is_empty (const char *path);
