@@ -67,8 +67,7 @@ static int rd_kafka_mock_handle_Produce (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, TopicsCnt);
 
         /* Inject error, if any */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         while (TopicsCnt-- > 0) {
                 rd_kafkap_str_t Topic;
@@ -197,8 +196,7 @@ static int rd_kafka_mock_handle_Fetch (rd_kafka_mock_connection_t *mconn,
 
 
         /* Inject error, if any */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (rkbuf->rkbuf_reqhdr.ApiVersion >= 7) {
                 /* Response: ErrorCode */
@@ -418,8 +416,7 @@ static int rd_kafka_mock_handle_ListOffset (rd_kafka_mock_connection_t *mconn,
 
 
         /* Inject error, if any */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         rd_kafka_buf_read_i32(rkbuf, &TopicsCnt);
 
@@ -551,8 +548,7 @@ static int rd_kafka_mock_handle_OffsetFetch (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_read_str(rkbuf, &GroupId);
 
         /* Inject error, if any */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         mrkb = rd_kafka_mock_cluster_get_coord(mcluster, RD_KAFKA_COORD_GROUP,
                                                &GroupId);
@@ -689,8 +685,7 @@ static int rd_kafka_mock_handle_OffsetCommit (rd_kafka_mock_connection_t *mconn,
 
 
         /* Inject error, if any */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         mrkb = rd_kafka_mock_cluster_get_coord(mcluster, RD_KAFKA_COORD_GROUP,
                                                &GroupId);
@@ -1045,8 +1040,7 @@ rd_kafka_mock_handle_FindCoordinator (rd_kafka_mock_connection_t *mconn,
         }
 
         /* Inject error, if any */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!err && RD_KAFKAP_STR_LEN(&Key) > 0) {
                 mrkb = rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1147,8 +1141,7 @@ rd_kafka_mock_handle_JoinGroup (rd_kafka_mock_connection_t *mconn,
         }
 
         /* Inject error, if any */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!err) {
                 mrkb = rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1235,8 +1228,7 @@ rd_kafka_mock_handle_Heartbeat (rd_kafka_mock_connection_t *mconn,
         }
 
         /* Inject error, if any */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
         if (!err) {
                 mrkb = rd_kafka_mock_cluster_get_coord(mcluster,
                                                        RD_KAFKA_COORD_GROUP,
@@ -1265,7 +1257,7 @@ rd_kafka_mock_handle_Heartbeat (rd_kafka_mock_connection_t *mconn,
                                                      GenerationId);
 
         if (!err)
-                rd_kafka_mock_cgrp_member_active(member);
+                rd_kafka_mock_cgrp_member_active(mcgrp, member);
 
         rd_kafka_buf_write_i16(resp, err); /* ErrorCode */
 
@@ -1307,8 +1299,7 @@ rd_kafka_mock_handle_LeaveGroup (rd_kafka_mock_connection_t *mconn,
         }
 
         /* Inject error, if any */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
         if (!err) {
                 mrkb = rd_kafka_mock_cluster_get_coord(mcluster,
                                                        RD_KAFKA_COORD_GROUP,
@@ -1368,7 +1359,6 @@ rd_kafka_mock_handle_SyncGroup (rd_kafka_mock_connection_t *mconn,
         rd_kafka_resp_err_t err;
         rd_kafka_mock_cgrp_t *mcgrp = NULL;
         rd_kafka_mock_cgrp_member_t *member = NULL;
-        rd_bool_t is_leader;
 
         rd_kafka_buf_read_str(rkbuf, &GroupId);
         rd_kafka_buf_read_i32(rkbuf, &GenerationId);
@@ -1386,8 +1376,7 @@ rd_kafka_mock_handle_SyncGroup (rd_kafka_mock_connection_t *mconn,
         }
 
         /* Inject error, if any */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
         if (!err) {
                 mrkb = rd_kafka_mock_cluster_get_coord(mcluster,
                                                        RD_KAFKA_COORD_GROUP,
@@ -1416,12 +1405,11 @@ rd_kafka_mock_handle_SyncGroup (rd_kafka_mock_connection_t *mconn,
                                                      GenerationId);
 
         if (!err)
-                rd_kafka_mock_cgrp_member_active(member);
-
-
-        is_leader = mcgrp->leader && mcgrp->leader == member;
+                rd_kafka_mock_cgrp_member_active(mcgrp, member);
 
         if (!err) {
+                rd_bool_t is_leader = mcgrp->leader && mcgrp->leader == member;
+
                 if (AssignmentCnt > 0 && !is_leader)
                         err = RD_KAFKA_RESP_ERR_NOT_LEADER_FOR_PARTITION; /* FIXME */
                 else if (AssignmentCnt == 0 && is_leader)
@@ -1475,10 +1463,74 @@ rd_kafka_mock_handle_SyncGroup (rd_kafka_mock_connection_t *mconn,
 /**
  * @brief Generate a unique ProducerID
  */
-static void rd_kafka_mock_pid_generate (rd_kafka_mock_cluster_t *mcluster,
-                                        rd_kafka_pid_t *pid) {
+static const rd_kafka_pid_t
+rd_kafka_mock_pid_new (rd_kafka_mock_cluster_t *mcluster) {
+        rd_kafka_pid_t *pid = rd_malloc(sizeof(*pid));
+        rd_kafka_pid_t ret;
+
         pid->id = rd_jitter(1, 900000) * 1000;
         pid->epoch = 0;
+
+        mtx_lock(&mcluster->lock);
+        rd_list_add(&mcluster->pids, pid);
+        ret = *pid;
+        mtx_unlock(&mcluster->lock);
+
+        return ret;
+}
+
+
+/**
+ * @brief Checks if the given pid is known, else returns an error.
+ */
+static rd_kafka_resp_err_t
+rd_kafka_mock_pid_check (rd_kafka_mock_cluster_t *mcluster,
+                         const rd_kafka_pid_t check_pid) {
+        const rd_kafka_pid_t *pid;
+        rd_kafka_resp_err_t err = RD_KAFKA_RESP_ERR_NO_ERROR;
+
+        mtx_lock(&mcluster->lock);
+        pid = rd_list_find(&mcluster->pids, &check_pid, rd_kafka_pid_cmp_pid);
+
+        if (!pid)
+                err = RD_KAFKA_RESP_ERR_UNKNOWN_PRODUCER_ID;
+        else if (check_pid.epoch != pid->epoch)
+                err = RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH;
+        mtx_unlock(&mcluster->lock);
+
+        return err;
+}
+
+
+/**
+ * @brief Bump the epoch for an existing pid, or return an error
+ *        if the current_pid does not match an existing pid.
+ */
+static rd_kafka_resp_err_t
+rd_kafka_mock_pid_bump (rd_kafka_mock_cluster_t *mcluster,
+                        rd_kafka_pid_t *current_pid) {
+        rd_kafka_pid_t *pid;
+
+        mtx_lock(&mcluster->lock);
+        pid = rd_list_find(&mcluster->pids, current_pid, rd_kafka_pid_cmp_pid);
+        if (!pid) {
+                mtx_unlock(&mcluster->lock);
+                return RD_KAFKA_RESP_ERR_UNKNOWN_PRODUCER_ID;
+        }
+
+        if (current_pid->epoch != pid->epoch) {
+                mtx_unlock(&mcluster->lock);
+                return RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH;
+        }
+
+        pid->epoch++;
+        *current_pid = *pid;
+        mtx_unlock(&mcluster->lock);
+
+        rd_kafka_dbg(mcluster->rk, MOCK, "MOCK",
+                     "Bumped PID %s", rd_kafka_pid2str(*current_pid));
+
+        return RD_KAFKA_RESP_ERR_NO_ERROR;
 }
 
 
@@ -1492,7 +1544,8 @@ rd_kafka_mock_handle_InitProducerId (rd_kafka_mock_connection_t *mconn,
         const rd_bool_t log_decode_errors = rd_true;
         rd_kafka_buf_t *resp = rd_kafka_mock_buf_new_response(rkbuf);
         rd_kafkap_str_t TransactionalId;
-        rd_kafka_pid_t pid;
+        rd_kafka_pid_t pid = RD_KAFKA_PID_INITIALIZER;
+        rd_kafka_pid_t current_pid = RD_KAFKA_PID_INITIALIZER;
         int32_t TxnTimeoutMs;
         rd_kafka_resp_err_t err;
 
@@ -1501,6 +1554,13 @@ rd_kafka_mock_handle_InitProducerId (rd_kafka_mock_connection_t *mconn,
         /* TransactionTimeoutMs */
         rd_kafka_buf_read_i32(rkbuf, &TxnTimeoutMs);
 
+        if (rkbuf->rkbuf_reqhdr.ApiVersion >= 3) {
+                /* ProducerId */
+                rd_kafka_buf_read_i64(rkbuf, &current_pid.id);
+                /* ProducerEpoch */
+                rd_kafka_buf_read_i16(rkbuf, &current_pid.epoch);
+        }
+
         /*
          * Construct response
          */
@@ -1509,24 +1569,37 @@ rd_kafka_mock_handle_InitProducerId (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, 0);
 
         /* Inject error */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
-        if (!err &&
-            !RD_KAFKAP_STR_IS_NULL(&TransactionalId) &&
-            rd_kafka_mock_cluster_get_coord(mcluster,
-                                            RD_KAFKA_COORD_TXN,
-                                            &TransactionalId) != mconn->broker)
-                err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
+        if (!err && !RD_KAFKAP_STR_IS_NULL(&TransactionalId)) {
+                if (RD_KAFKAP_STR_LEN(&TransactionalId) == 0)
+                        err = RD_KAFKA_RESP_ERR_INVALID_REQUEST;
+                else if (rd_kafka_mock_cluster_get_coord(
+                                 mcluster, RD_KAFKA_COORD_TXN,
+                                 &TransactionalId) != mconn->broker)
+                        err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
+        }
+
+        if (!err) {
+                if (rd_kafka_pid_valid(current_pid)) {
+                        /* Producer is asking for the transactional coordinator
+                         * to bump the epoch (KIP-360).
+                         * Verify that current_pid matches and then
+                         * bump the epoch. */
+                        err = rd_kafka_mock_pid_bump(mcluster,
+                                                     &current_pid);
+                        if (!err)
+                                pid = current_pid;
+
+                } else {
+                        /* Generate a new pid */
+                        pid = rd_kafka_mock_pid_new(mcluster);
+                }
+        }
 
         /* ErrorCode */
         rd_kafka_buf_write_i16(resp, err);
 
-        if (!err)
-                rd_kafka_mock_pid_generate(mcluster, &pid);
-        else
-                rd_kafka_pid_reset(&pid);
-
         /* ProducerId */
         rd_kafka_buf_write_i64(resp, pid.id);
         /* ProducerEpoch */
@@ -1573,8 +1646,7 @@ rd_kafka_mock_handle_AddPartitionsToTxn (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, TopicsCnt);
 
         /* Inject error */
-        all_err = rd_kafka_mock_next_request_error(mconn,
-                                                   rkbuf->rkbuf_reqhdr.ApiKey);
+        all_err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!all_err &&
             rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1582,6 +1654,9 @@ rd_kafka_mock_handle_AddPartitionsToTxn (rd_kafka_mock_connection_t *mconn,
                                             &TransactionalId) != mconn->broker)
                 all_err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
 
+        if (!all_err)
+                all_err = rd_kafka_mock_pid_check(mcluster, pid);
+
         while (TopicsCnt-- > 0) {
                 rd_kafkap_str_t Topic;
                 int32_t PartsCnt;
@@ -1655,8 +1730,7 @@ rd_kafka_mock_handle_AddOffsetsToTxn (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, 0);
 
         /* Inject error */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!err &&
             rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1664,6 +1738,9 @@ rd_kafka_mock_handle_AddOffsetsToTxn (rd_kafka_mock_connection_t *mconn,
                                             &TransactionalId) != mconn->broker)
                 err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
 
+        if (!err)
+                err = rd_kafka_mock_pid_check(mcluster, pid);
+
         /* Response: ErrorCode */
         rd_kafka_buf_write_i16(resp, err);
 
@@ -1709,8 +1786,7 @@ rd_kafka_mock_handle_TxnOffsetCommit (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, TopicsCnt);
 
         /* Inject error */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!err &&
             rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1718,6 +1794,9 @@ rd_kafka_mock_handle_TxnOffsetCommit (rd_kafka_mock_connection_t *mconn,
                                             &GroupId) != mconn->broker)
                 err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
 
+        if (!err)
+                err = rd_kafka_mock_pid_check(mcluster, pid);
+
         while (TopicsCnt-- > 0) {
                 rd_kafkap_str_t Topic;
                 int32_t PartsCnt;
@@ -1802,8 +1881,7 @@ rd_kafka_mock_handle_EndTxn (rd_kafka_mock_connection_t *mconn,
         rd_kafka_buf_write_i32(resp, 0);
 
         /* Inject error */
-        err = rd_kafka_mock_next_request_error(mconn,
-                                               rkbuf->rkbuf_reqhdr.ApiKey);
+        err = rd_kafka_mock_next_request_error(mconn, resp);
 
         if (!err &&
             rd_kafka_mock_cluster_get_coord(mcluster,
@@ -1811,6 +1889,9 @@ rd_kafka_mock_handle_EndTxn (rd_kafka_mock_connection_t *mconn,
                                             &TransactionalId) != mconn->broker)
                 err = RD_KAFKA_RESP_ERR_NOT_COORDINATOR;
 
+        if (!err)
+                err = rd_kafka_mock_pid_check(mcluster, pid);
+
         /* ErrorCode */
         rd_kafka_buf_write_i16(resp, err);
 
@@ -1829,6 +1910,7 @@ rd_kafka_mock_handle_EndTxn (rd_kafka_mock_connection_t *mconn,
  */
 const struct rd_kafka_mock_api_handler
 rd_kafka_mock_api_handlers[RD_KAFKAP__NUM] = {
+        /* [request-type] = { MinVersion, MaxVersion, FlexVersion, callback } */
         [RD_KAFKAP_Produce] = { 0, 7, -1, rd_kafka_mock_handle_Produce },
         [RD_KAFKAP_Fetch] = { 0, 11, -1, rd_kafka_mock_handle_Fetch },
         [RD_KAFKAP_Offset] = { 0, 5, -1, rd_kafka_mock_handle_ListOffset },
@@ -1839,7 +1921,7 @@ rd_kafka_mock_api_handlers[RD_KAFKAP__NUM] = {
         [RD_KAFKAP_Metadata] = { 0, 2, 9, rd_kafka_mock_handle_Metadata },
         [RD_KAFKAP_FindCoordinator] = { 0, 2, 3,
                                         rd_kafka_mock_handle_FindCoordinator },
-        [RD_KAFKAP_InitProducerId] = { 0, 1, 2,
+        [RD_KAFKAP_InitProducerId] = { 0, 4, 2,
                                        rd_kafka_mock_handle_InitProducerId },
         [RD_KAFKAP_JoinGroup] = { 0, 5, 6, rd_kafka_mock_handle_JoinGroup },
         [RD_KAFKAP_Heartbeat] = { 0, 3, 4, rd_kafka_mock_handle_Heartbeat },
@@ -1875,6 +1957,11 @@ static int rd_kafka_mock_handle_ApiVersion (rd_kafka_mock_connection_t *mconn,
                     rkbuf->rkbuf_reqhdr.ApiKey, rkbuf->rkbuf_reqhdr.ApiVersion))
                 err = RD_KAFKA_RESP_ERR_UNSUPPORTED_VERSION;
 
+        /* ApiVersionRequest/Response with flexver (>=v3) has a mix
+         * of flexver and standard fields for backwards compatibility reasons,
+         * so we handcraft the response instead. */
+        resp->rkbuf_flags &= ~RD_KAFKA_OP_F_FLEXVER;
+
         /* ErrorCode */
         rd_kafka_buf_write_i16(resp, err);
 
