@@ -307,14 +307,14 @@ static void s3_context_destroy(struct flb_s3 *ctx)
         flb_tls_destroy(ctx->sts_provider_tls);
     }
 
-    if (ctx->client_tls) {
-        flb_tls_destroy(ctx->client_tls);
-    }
-
     if (ctx->s3_client) {
         flb_aws_client_destroy(ctx->s3_client);
     }
 
+    if (ctx->client_tls) {
+        flb_tls_destroy(ctx->client_tls);
+    }
+
     if (ctx->free_endpoint == FLB_TRUE) {
         flb_free(ctx->endpoint);
     }
@@ -1165,12 +1165,13 @@ int get_md5_base64(char *buf, size_t buf_size, char *md5_str, size_t md5_str_siz
     size_t olen;
     int ret;
 
-    ret = mbedtls_md5_ret(buf, buf_size, md5_bin);
+    ret = mbedtls_md5_ret((unsigned char*) buf, buf_size, md5_bin);
     if (ret != 0) {
         return ret;
     }
 
-    ret = mbedtls_base64_encode(md5_str, md5_str_size, &olen, md5_bin, sizeof(md5_bin));
+    ret = mbedtls_base64_encode((unsigned char*) md5_str, md5_str_size, &olen, md5_bin,
+                                sizeof(md5_bin));
     if (ret != 0) {
         return ret;
     }
