@@ -43,12 +43,6 @@
 pthread_key_t oauth2_type;
 pthread_key_t oauth2_token;
 
-int is_tag_match_regex(struct flb_stackdriver *ctx, const char *tag, int tag_len);
-int is_local_resource_id_match_regex(struct flb_stackdriver *ctx);
-int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
-                                       const char *tag, int tag_len,
-                                       int from_tag);
-
 static void oauth2_cache_exit(void *ptr)
 {
     if (ptr) {
@@ -66,24 +60,23 @@ static void oauth2_cache_init()
 /* Set oauth2 type and token in pthread keys */
 static void oauth2_cache_set(char *type, char *token)
 {
-    flb_sds_t buf_type;
-    flb_sds_t buf_token;
+    flb_sds_t tmp;
 
     /* oauth2 type */
-    buf_type = pthread_getspecific(oauth2_type);
-    if (buf_type) {
-        flb_sds_destroy(buf_type);
+    tmp = pthread_getspecific(oauth2_type);
+    if (tmp) {
+        flb_sds_destroy(tmp);
     }
-    buf_type = flb_sds_create(type);
-    pthread_setspecific(oauth2_type, buf_type);
+    tmp = flb_sds_create(type);
+    pthread_setspecific(oauth2_type, tmp);
 
     /* oauth2 access token */
-    buf_token = pthread_getspecific(oauth2_token);
-    if (buf_token) {
-        flb_sds_destroy(buf_token);
+    tmp = pthread_getspecific(oauth2_token);
+    if (tmp) {
+        flb_sds_destroy(tmp);
     }
-    buf_token = flb_sds_create(token);
-    pthread_setspecific(oauth2_token, buf_token);
+    tmp = flb_sds_create(token);
+    pthread_setspecific(oauth2_token, tmp);
 }
 
 /* By using pthread keys cached values, compose the authorizatoin token */
@@ -810,6 +803,95 @@ static int set_monitored_resource_labels(struct flb_stackdriver *ctx, char *type
     return -1;
 }
 
+static int is_tag_match_regex(struct flb_stackdriver *ctx,
+                              const char *tag, int tag_len)
+{
+    int ret;
+    int tag_prefix_len;
+    int len_to_be_matched;
+    const char *tag_str_to_be_matcheds;
+
+    tag_prefix_len = flb_sds_len(ctx->tag_prefix);
+    if (tag_len > tag_prefix_len &&
+        flb_sds_cmp(ctx->tag_prefix, tag, tag_prefix_len) != 0) {
+        return 0;
+    }
+
+    tag_str_to_be_matcheds = tag + tag_prefix_len;
+    len_to_be_matched = tag_len - tag_prefix_len;
+    ret = flb_regex_match(ctx->regex,
+                          (unsigned char *) tag_str_to_be_matcheds,
+                          len_to_be_matched);
+
+    /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
+    return ret;
+}
+
+static int is_local_resource_id_match_regex(struct flb_stackdriver *ctx)
+{
+    int ret;
+    int prefix_len;
+    int len_to_be_matched;
+    const char *str_to_be_matcheds;
+
+    if (!ctx->local_resource_id) {
+        flb_plg_warn(ctx->ins, "local_resource_id not found in the payload");
+        return -1;
+    }
+
+    prefix_len = flb_sds_len(ctx->tag_prefix);
+    str_to_be_matcheds = ctx->local_resource_id + prefix_len;
+    len_to_be_matched = flb_sds_len(ctx->local_resource_id) - prefix_len;
+
+    ret = flb_regex_match(ctx->regex,
+                          (unsigned char *) str_to_be_matcheds,
+                          len_to_be_matched);
+
+    /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
+    return ret;
+}
+
+static void cb_results(const char *name, const char *value,
+                       size_t vlen, void *data);
+/*
+ * extract_resource_labels_from_regex(4) will only be called if the
+ * tag or local_resource_id field matches the regex rule
+ */
+static int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
+                                              const char *tag, int tag_len,
+                                              int from_tag)
+{
+    int ret = 1;
+    int prefix_len;
+    int len_to_be_matched;
+    int local_resource_id_len;
+    const char *str_to_be_matcheds;
+    struct flb_regex_search result;
+
+    prefix_len = flb_sds_len(ctx->tag_prefix);
+    if (from_tag == FLB_TRUE) {
+        local_resource_id_len = tag_len;
+        str_to_be_matcheds = tag + prefix_len;
+    }
+    else {
+        // this will be called only if the payload contains local_resource_id
+        local_resource_id_len = flb_sds_len(ctx->local_resource_id);
+        str_to_be_matcheds = ctx->local_resource_id + prefix_len;
+    }
+
+    len_to_be_matched = local_resource_id_len - prefix_len;
+    ret = flb_regex_do(ctx->regex, str_to_be_matcheds, len_to_be_matched, &result);
+    if (ret <= 0) {
+        flb_plg_warn(ctx->ins, "invalid pattern for given value %s when"
+                     " extracting resource labels", str_to_be_matcheds);
+        return -1;
+    }
+
+    flb_regex_parse(ctx->regex, &result, cb_results, ctx);
+
+    return ret;
+}
+
 static int process_local_resource_id(struct flb_stackdriver *ctx,
                                      const char *tag, int tag_len, char *type)
 {
@@ -914,88 +996,6 @@ int flb_stackdriver_regex_init(struct flb_stackdriver *ctx)
     return 0;
 }
 
-int is_tag_match_regex(struct flb_stackdriver *ctx, const char *tag, int tag_len)
-{
-    int ret;
-    int tag_prefix_len;
-    int len_to_be_matched;
-    const char *tag_str_to_be_matcheds;
-
-    tag_prefix_len = flb_sds_len(ctx->tag_prefix);
-    tag_str_to_be_matcheds = tag + tag_prefix_len;
-    len_to_be_matched = tag_len - tag_prefix_len;
-
-    ret = flb_regex_match(ctx->regex,
-                          (unsigned char *) tag_str_to_be_matcheds,
-                          len_to_be_matched);
-
-    /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
-    return ret;
-}
-
-int is_local_resource_id_match_regex(struct flb_stackdriver *ctx)
-{
-    int ret;
-    int prefix_len;
-    int len_to_be_matched;
-    const char *str_to_be_matcheds;
-
-    if (!ctx->local_resource_id) {
-        flb_plg_warn(ctx->ins, "local_resource_id not found in the payload");
-        return -1;
-    }
-
-    prefix_len = flb_sds_len(ctx->tag_prefix);
-    str_to_be_matcheds = ctx->local_resource_id + prefix_len;
-    len_to_be_matched = flb_sds_len(ctx->local_resource_id) - prefix_len;
-
-    ret = flb_regex_match(ctx->regex,
-                          (unsigned char *) str_to_be_matcheds,
-                          len_to_be_matched);
-
-    /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
-    return ret;
-}
-
-/*
- * extract_resource_labels_from_regex(4) will only be called if the
- * tag or local_resource_id field matches the regex rule
- */
-int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
-                                       const char *tag, int tag_len,
-                                       int from_tag)
-{
-    int ret = 1;
-    int prefix_len;
-    int len_to_be_matched;
-    int local_resource_id_len;
-    const char *str_to_be_matcheds;
-    struct flb_regex_search result;
-
-    prefix_len = flb_sds_len(ctx->tag_prefix);
-    if (from_tag == FLB_TRUE) {
-        local_resource_id_len = tag_len;
-        str_to_be_matcheds = tag + prefix_len;
-    }
-    else {
-        // this will be called only if the payload contains local_resource_id
-        local_resource_id_len = flb_sds_len(ctx->local_resource_id);
-        str_to_be_matcheds = ctx->local_resource_id + prefix_len;
-    }
-
-    len_to_be_matched = local_resource_id_len - prefix_len;
-    ret = flb_regex_do(ctx->regex, str_to_be_matcheds, len_to_be_matched, &result);
-    if (ret <= 0) {
-        flb_plg_warn(ctx->ins, "invalid pattern for given value %s when"
-                     " extracting resource labels", str_to_be_matcheds);
-        return -1;
-    }
-
-    flb_regex_parse(ctx->regex, &result, cb_results, ctx);
-
-    return ret;
-}
-
 static int cb_stackdriver_init(struct flb_output_instance *ins,
                           struct flb_config *config, void *data)
 {
