@@ -702,7 +702,7 @@ static flb_sds_t syslog_format(struct flb_syslog *ctx, msgpack_object *o,
             msg.facility = 1;
         }
 
-        if (ctx->format == FLB_SYSLOG_RFC3164) {
+        if (ctx->parsed_format == FLB_SYSLOG_RFC3164) {
             tmp = syslog_rfc3164(s, tm, &msg);
         }
         else {
@@ -851,7 +851,7 @@ static int cb_syslog_init(struct flb_output_instance *ins, struct flb_config *co
     }
 
     if (ctx->maxsize < 0) {
-        if (ctx->format == FLB_SYSLOG_RFC3164) {
+        if (ctx->parsed_format == FLB_SYSLOG_RFC3164) {
             ctx->maxsize = RFC3164_MAXSIZE;
         }
         else {
