@@ -43,6 +43,7 @@
 #include <fluent-bit/flb_sosreport.h>
 #include <fluent-bit/flb_storage.h>
 #include <fluent-bit/flb_http_server.h>
+#include <cmetrics/cmetrics.h>
 
 #ifdef FLB_HAVE_METRICS
 #include <fluent-bit/flb_metrics_exporter.h>
@@ -52,6 +53,12 @@
 #include <fluent-bit/stream_processor/flb_sp.h>
 #endif
 
+#ifdef FLB_HAVE_AWS_ERROR_REPORTER
+#include <fluent-bit/aws/flb_aws_error_reporter.h>
+
+extern struct flb_aws_error_reporter *error_reporter;
+#endif
+
 FLB_TLS_DEFINE(struct mk_event_loop, flb_engine_evl);
 
 
@@ -466,13 +473,6 @@ int flb_engine_start(struct flb_config *config)
     /* Initialize the networking layer */
     flb_net_init();
 
-    /* HTTP Server */
-#ifdef FLB_HAVE_HTTP
-    if (config->http_server == FLB_TRUE) {
-        flb_http_server_start(config);
-    }
-#endif
-
     /* Create the event loop and set it in the global configuration */
     evl = mk_event_loop_create(256);
     if (!evl) {
@@ -518,6 +518,22 @@ int flb_engine_start(struct flb_config *config)
         return -1;
     }
 
+    /* Init Metrics engine */
+    cmt_initialize();
+    flb_info("[cmetrics] version=%s", cmt_version());
+
+    /* Initialize the scheduler */
+    sched = flb_sched_create(config, config->evl);
+    if (!sched) {
+        flb_error("[engine] scheduler could not start");
+        return -1;
+    }
+    config->sched = sched;
+
+    /* Register the scheduler context */
+    flb_sched_ctx_init();
+    flb_sched_ctx_set(sched);
+
     /* Initialize input plugins */
     ret = flb_input_init_all(config);
     if (ret == -1) {
@@ -556,17 +572,6 @@ int flb_engine_start(struct flb_config *config)
         flb_utils_error(FLB_ERR_CFG_FLUSH_CREATE);
     }
 
-    /* Initialize the scheduler */
-    sched = flb_sched_create(config, config->evl);
-    if (!sched) {
-        flb_error("[engine] scheduler could not start");
-        return -1;
-    }
-    config->sched = sched;
-
-    /* Register the scheduler context */
-    flb_sched_ctx_init();
-    flb_sched_ctx_set(sched);
 
 #ifdef FLB_HAVE_METRICS
     if (config->storage_metrics == FLB_TRUE) {
@@ -710,6 +715,16 @@ int flb_engine_start(struct flb_config *config)
         if (config->is_running == FLB_TRUE) {
             flb_sched_timer_cleanup(config->sched);
             flb_upstream_conn_pending_destroy_list(&config->upstreams);
+
+            /*
+            * depend on main thread to clean up expired message
+            * in aws error reporting message queue
+            */
+            #ifdef FLB_HAVE_AWS_ERROR_REPORTER
+            if (is_error_reporting_enabled()) {
+                flb_aws_error_reporter_clean(error_reporter);
+            }
+            #endif
         }
     }
 }
@@ -730,11 +745,6 @@ int flb_engine_shutdown(struct flb_config *config)
     /* router */
     flb_router_exit(config);
 
-#ifdef FLB_HAVE_PARSER
-    /* parsers */
-    flb_parser_exit(config);
-#endif
-
     /* cleanup plugins */
     flb_filter_exit(config);
     flb_input_exit_all(config);
@@ -756,8 +766,6 @@ int flb_engine_shutdown(struct flb_config *config)
     }
 #endif
 
-    flb_config_exit(config);
-
     return 0;
 }
 
